import sys, logging

try:
    from win32com.client import gencache, DispatchWithEvents
    import pythoncom, win32com
  
    from win32com.axcontrol import axcontrol  # provides IObjectWithSite

except ImportError, e:
    print '[!] Could not import required module!:\n\t%s' % e
    sys.exit(1)


class PythonBHO:
    '''This is the Browser Helper Object wrapper which receives SetSite event 
    on MSIE start and registers event handlers
    '''
  
    _reg_clsid_ = "{0bcaa6f0-c397-11db-abbd-0800200c9a66}"
    _reg_progid_ = "PythonBHO"
    _public_methods_ = ['SetSite', 'GetSite']
  
    #  the line below is needed for DLL version only
    _com_interfaces_ = [axcontrol.IID_IObjectWithSite]  # from iebutton.py    
# snip snip snip

# some other code was here...
        
# snip snip snip
      
##########################
# BHO registration stuff #
##########################


def RegisterBHO(klass):
    '''registers BHO with MSIE (adds record to registry)'''
  
    try:
        import _winreg  # lazy import
    except ImportError, e:
        print '[!] Could not import _winreg!:\n\t%s' % e
        logging.error('ERROR: Could not import _winreg: %s' % e)
  
    try: 
        key = _winreg.CreateKey(_winreg.HKEY_LOCAL_MACHINE, 
                                "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Browser Helper Objects\\")
        _winreg.CreateKey(key, klass._reg_clsid_)
        if debug: 
            print '[d] CreateKey completed, BHO succesfully registered'
    except WindowsError, e:
        logging.error('Could not register BHO!')
        if debug: 
            print '[!] Could not register BHO!\n\t%s' % e 


def UnregisterBHO(klass):
    '''unregisters BHO (deletes entry from registry'''
  
    try:
        import _winreg  # lazy import
    except ImportError, e:
        print '[!] Could not import _winreg!:\n\t%s' % e
        logging.error('ERROR: Could not import _winreg: %s' % e)
  
    try:
        _winreg.DeleteKey(_winreg.HKEY_LOCAL_MACHINE, 
                          "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Browser Helper Objects\\" + klass._reg_clsid_)
    except WindowsError, e:
        logging.error('Could not unregister BHO!')
        if debug: 
            print '[!] Could not unregister BHO!\n\t%s' % e 


# this is used when DLL is registered using regsvr32
def DllRegisterServer():
    '''regsvr32 handler, calls RegisterBHO'''
    
    print 'DLLRegisterServer entered'
    klass = PythonBHO
    RegisterBHO(klass)
    print 'Registration complete'  # we want to use 'print' here


# this is used when DLL is unregistered by regsvr32 /u 
def DllUnregisterServer():
    '''regsvr32 /u handler, calls UnregisterBHO'''
  
    klass = PythonBHO
    UnregisterBHO(klass)
    print 'Unregistered'  # we want to use 'print' here


if __name__ == '__main__':
    try:
        import win32com.server.register  # lazy import
    except ImportError, e:
        print '[!] Could not import win32com.server.register!:\n\t%s' % e
        logging.error('ERROR: Could not import _winreg: %s' % e)
    
    win32com.server.register.UseCommandLine(PythonBHO)
    if "--unregister" in sys.argv: 
        UnregisterBHO(PythonBHO)
    else:
        RegisterBHO(PythonBHO)
